package com.chelvc.framework.base.interceptor;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.web.servlet.error.ErrorController;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.ExceptionHandler;

/**
 * Controller接口访问拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Aspect
@Component
@Order(Ordered.HIGHEST_PRECEDENCE)
public class ControllerAccessInterceptor {
    /**
     * 判断是否开启调试日志
     *
     * @param point 接口方法拦截点
     * @return true/false
     */
    private boolean isDebug(JoinPoint point) {
        Object target = point.getTarget();
        if (target instanceof ErrorController
                || !ApplicationContextHolder.getProperty("controller.debug.enabled", boolean.class, false)) {
            return false;
        }
        Signature signature = point.getSignature();
        return signature instanceof MethodSignature
                && !((MethodSignature) signature).getMethod().isAnnotationPresent(ExceptionHandler.class);
    }

    /**
     * 接口请求日志打印
     *
     * @param point 接口方法拦截点
     * @throws Throwable 接口调用异常
     */
    @Around("@within(org.springframework.stereotype.Controller) " +
            "|| @within(org.springframework.web.bind.annotation.RestController)")
    public Object logging(ProceedingJoinPoint point) throws Throwable {
        Object value = point.proceed();
        if (this.isDebug(point)) {
            Logger logger = LoggerFactory.getLogger(point.getSignature().getDeclaringTypeName());
            LoggingContextHolder.debug(logger, SessionContextHolder.getRequest(), value);
        }
        return value;
    }
}
